#ifndef DG036Z_H
#define DG036Z_H

#include <sys/io.h>

#include <libtools.h>

#ifndef byte
typedef unsigned char byte;
#endif

class BITARRAY
{
  public :
    enum OP
    {
      CLEAR,
      SET,
      XOR
    };

          BITARRAY    (int Width, int Height);
         ~BITARRAY    ();

   void   Clear       (bool Bit = 0);
   void   SetBit      (int x, int y, bool Bit, OP Operation = SET);
   bool   GetBit      (int x, int y);
   byte   GetByte     (int x, int y, int Base = 8);

  protected :
    int   GetIndex    (int x, int y, int Base = 8);

  private :
    int   SizeX, SizeY;
    int   Cols;
    byte *Data;    
};


/* ********************************************************************** */ 
/* ** Class of Toshiba T6963C display.                                 ** */
/* ** TxtData contains the characters of the display's text page       ** */
/* ** ImgData are the pixels of the graphical page                     ** */
/* ** Panel contains the actual displayed image. The data will be      ** */
/* ** created when calling the Refresh-method and can be read out to   ** */
/* ** display simulation.                                              ** */
/* ********************************************************************** */
class T6963C
{
  public :
    enum OP
    {
      SAME = -1,
      CLEAR,
      SET,
      SWAP
    };

    enum MODE
    {
      OFF          = 0,
      CURSOR       = (1 << 0),
      CURSOR_BLINK = (1 << 1),
      TEXT         = (1 << 2),
      GRAPHIC      = (1 << 3)
    };

           T6963C            (unsigned short Port, 
			      int Width = 120, int Height = 48);
          ~T6963C            ();

    void      Clear             (MODE Mode);
    void      Clear             (int x1 = -1, int y1 = -1, 
				 int x2 = -1, int y2 = -1);
    void      FadeOut           (int Mode);
    void      Print             (int PosX, int PosY, const char *Text);
    void      ShowImage         (unsigned char *Data, int SizeX, int SizeY);
    void      ShowText          (const char *Text);
    void      SetCursorPattern  (int No);
    void      SetCursorPos      (int PosX, int PosY);
    void      SetMode           (int DispMode);

    int       GetMode           () const;
    BITARRAY *GetImage          () const;
    void      Refresh           ();
    void      SetRefreshMode    (int Mode);

    void      Transfer          ();

    void      DrawArc           (int x, int y, int w, int h, 
                                 int Angle1 = 0, int Angle2 = 360, 
				 OP Pixel = SET);
    void      DrawLine          (int PosX1, int PosY1, int PosX2, int PosY2,
				 OP Pixel = SET);
    void      DrawPoint         (int PosX, int PosY, OP Operation = SET);
    void      DrawRectangle     (int x1, int y1, int x2, int y2, 
				 OP Pixel = SET);

    void      FillArc           (int x, int y, int w, int h, 
                                 int Angle1 = 0, int Angle2 = 360, 
				 OP Pixel = SET, OP Fill = SAME);
    void      FillPolygon       (XPoint *Points, int Number, 
				 int Shape = Complex, 
				 int Mode = CoordModeOrigin, 
				 OP Pixel = SET);
    void      FillRectangle     (int x1, int y1, int x2, int y2, 
				 OP Pixel = SET, OP Fill = SAME);

    void      Move              (int OffsX, int OffsY,
				 int x1 = -1, int y1 = -1, 
				 int x2 = -1, int y2 = -1);

  protected :
    void      ClearText         ();
    void      ClearImage        ();
    byte      GetCode           (char Letter);
    void      RefreshHardware   ();
    void      RefreshPanel      ();
    void      SetAddressPointer (unsigned short Address);
    void      SetImage          (unsigned char *Data, int SizeX, int SizeY);
    void      PutChar           (int PosX, int PosY, 
				 const unsigned char Letter);
    void      SetText           (const char *Text);

  private :
    enum DISP_STATUS
    {
      STATUS_BAD,
      STATUS_OK,
      STATUS_AUTO_RD,
      STATUS_AUTO_WR
    };
  
    byte        GetByte      (int x, int y);

    int         CheckStatus  (byte Mask); 
    void        SendCmd      (unsigned short Port, byte Value);
    void        SendData     (unsigned short Port, byte Value);
    void        Cmd0         (byte Cmd, byte Mask);
    void        Cmd1         (byte Cmd, byte Param1, byte Mask);
    void        Cmd2         (byte Cmd, byte Param1, byte Param2, byte Mask);
    void        Data0        (byte Data, byte Mask);
 
    unsigned short      LPTBase;
             int        SizeX, SizeY;  // Size [pixel]
             byte       Rows, Cols;    // Size [char]
             byte       Mode;          // Text, Graphic, Text+Graphic
	     byte       FontX, FontY;
	     int        RefreshMode;
    unsigned short      TextHome, GraphicHome;

             byte     **TxtData;       // data of text screen
#if 0
	     byte     **ImgData;       // data of image screen
#endif
             BITARRAY  *Panel;       // Panel output

	     Pixmap     PixMap;
	     GC         PanelGC, PanelXOR;
};


class DG036Z : public T6963C
{
  public :
    DG036Z (unsigned short Port);
};


#endif
